// Author: James Kovacs [jkovacs@post.harvard.edu]
#region Copyright
// Copyright  2004 James Kovacs
// All rights reserved.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
// EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
// MERCHANTIBILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
#endregion
#region History
// 6/18/03: Initial release
#endregion

using System;
using System.Collections;
using System.IO;

namespace ZoneStripper {
	/// <summary>
	/// Summary description for Stripper.
	/// </summary>
	public class Stripper {
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) {
			try {
				if(args.Length == 0 || args[0] == "-h" || args[0] == "/?" || args[0] == "?") {
					PrintUsage();
					return;
				}

				Stripper program = new Stripper(args);
				program.Run();
			} catch(Exception ex) {
				Console.WriteLine(ex.Message);
			}
		}

		public Stripper(string[] args) {
			ParseArguments(args);
		}

		private void ParseArguments(string[] args) {
			m_recurse = false;
			m_dirs = new ArrayList();
			m_files = new ArrayList();
			foreach(string s in args) {
				if(s == "-r") {
					m_recurse = true;
				} else {
					string[] entries;
					string entryName = Path.GetFileName(s);	// could contain wildcards, filename, or directory name
					string parentName = Path.GetDirectoryName(s);
					if(parentName.IndexOfAny(WILDCARDS) > 0) {
						throw new ArgumentException("Wildcards are not allowed in pathnames.");
					}
					if(entryName.IndexOfAny(WILDCARDS) >= 0) {
						entries = Directory.GetFileSystemEntries(parentName, entryName);
					} else {
						entries = Directory.GetFileSystemEntries(s);
					}
					foreach(string entry in entries) {
						if(Directory.Exists(entry)) {
							m_dirs.Add(Path.GetFullPath(entry));
						} else if(File.Exists(entry)) {
							m_files.Add(Path.GetFullPath(entry));
						} else {
							throw new ArgumentException("Unknown file or directory, " + s);
						}
					}
				}
			}
		}

		public void Run() {
			foreach(string file in m_files) {
				WipeZoneId(file);
			}
			foreach(string dir in m_dirs) {
				WipeZoneId(dir);
				if(m_recurse) {
					RecurseDirectory(dir);
				}
			}
		}

		private void RecurseDirectory(string dirName) {
			DirectoryInfo di = new DirectoryInfo(dirName);
			foreach(FileInfo file in di.GetFiles()) {
				WipeZoneId(file.FullName);
			}
			foreach(DirectoryInfo subdir in di.GetDirectories()) {
				WipeZoneId(subdir.FullName);
				RecurseDirectory(subdir.FullName);
			}
		}

		private void WipeZoneId(string fullPathName) {
			try {
				if(Win32.StreamExists(fullPathName, ZONE_IDENTIFIER)) {
					Win32.DeleteStream(fullPathName, ZONE_IDENTIFIER);
				} 
			} catch(UnauthorizedAccessException ex) {
				Console.WriteLine(string.Format("Unable to remove Zone Identifier from {0}: {1}", fullPathName, ex.Message));
			}
		}

		private static void PrintUsage() {
			Console.WriteLine("ZoneStripper [-r] names");
			Console.WriteLine("where -r optionally recurses subdirectories (ignored if only files listed)");
			Console.WriteLine("  and names is a space-separated list of directories and/or files");
			Console.WriteLine("ZoneStripper {-h|/?|?} displays this message");
		}	

		private bool m_recurse;
		private ArrayList m_dirs;
		private ArrayList m_files;
		private const string ZONE_IDENTIFIER = "Zone.Identifier";
		private static readonly char[] WILDCARDS = new char[] {'*', '?'};
	}
}
